import React, { useEffect, useMemo, useRef, useState } from "react";
import {
  CloseOutlined,
  FieldTimeOutlined,
  SendOutlined,
  EditOutlined,
  DeleteOutlined,
  CaretDownOutlined,
} from "@ant-design/icons";
import { Button, Tooltip, Input, Form, Dropdown, MenuProps } from "antd";
import type { DropDownProps } from "antd";
import { useChat } from "@/hooks/useChat";


interface ChatHistoryItem {
  id: string;
  messages: any[];
  createdAt: number;
  updatedAt: number;
  title: string;
}

const dropDownMenu: DropDownProps['menu'] = {
  items: [
    { key: "1", label: "流程图" },
    { key: "2", label: "泳道图" },
    { key: "3", label: "ER实体图" },
    { key: "4", label: "组织图" },
    { key: "5", label: "时序图" },
  ]
}

export default function AICreator(props: { 
  type: 'mindmap' | 'flow',
  onClose?: () => void,
  onChange?: (data: any) => void,
}) {
  const [focused, setFocused] = React.useState(false);
  const [chatStarted, setChatStarted] = React.useState(false);
  const scrollAreaRef = React.useRef<HTMLDivElement>(null);
  const [input, setInput] = useState("");

  const {
    messages,
    setMessages,
    loading,
    onRequest
  } = useChat({
    app_name: "app1",
    onStart: () => {

    },
    onUpdate: (msg) => {
      setMessages((messages) => {
        const arr = [...messages];
        arr[messages.length - 1].content += msg.answer;
        arr[messages.length - 1].id = msg.message_id;
        return arr;
      });
    },
    onSuccess: (msg) => {
      console.log('加载完毕！');
    },
  })

  // 处理提交
  const onSubmit = () => {
    if (input.trim()) {

      if (!chatStarted) setChatStarted(true);
    }
  };

  React.useEffect(() => {
    return () => {
      // 取消所有进行中的请求
      const controller = new AbortController();
      controller.abort();
    };
  }, []);

  const [hoverId, setHoverId] = useState<string | null>(null);

    // 获取items
    const getItems = (list: ChatHistoryItem[]) => {
      return (list || []).map((item) => {
        return {
          key: item.id,
          label: (
            <div className="w-180px relative">
              <div className="w-full flex">
                <span
                  className="truncate"
                  style={{
                    overflow: "hidden",
                    whiteSpace: "nowrap",
                    textOverflow: "ellipsis",
                  }}
                >
                  <Tooltip title={item.title}>{item.title}</Tooltip>
                </span>
              </div>
              {/* TODO: 添加编辑删除按钮 */}
              {hoverId === item.id && (
                <div className="h-full w-50px text-right absolute right-0 top-0 bg-#fff">
                  <EditOutlined
                    onClick={(e) => {
                      e.stopPropagation();
                    }}
                  />
                  <DeleteOutlined />
                </div>
              )}
            </div>
          ),
          onClick: () => {
            
          },
          onMouseOver: () => {
            
          },
        };
      });
    };

  const handleList = [
    {
      key: "1",
      label: "风格美化",
      icon: "icon-yijianmeihua",
      color: "#a171f2",
    },
    {
      key: "2",
      label: "语法修复",
      icon: "icon-tubiao_yufajiucuo",
      color: "#00c4ad",
    },
    {
      key: "3",
      label: "翻译为英文",
      icon: "icon-fanyiweiyingwen",
      color: "#8c4ff0",
    },
    {
      key: "4",
      label: "翻译为中文",
      icon: "icon-fanyiweizhongwen",
      color: "#3d72fb",
    },
  ];

  return (
    <div className="flex-1 h-full">
      <div className="chat-head w-full h-40px px-10px color-#333 flex items-center justify-between">
        <i className="iconfont icon-AIchuangzuo"></i>
        <span>
          <Dropdown
            menu={{  }}
            trigger={["click"]}
            placement="bottomLeft"
            arrow
          >
            <Tooltip title="历史记录">
              <Button
                type="text"
                size="small"
                icon={<FieldTimeOutlined />}
              ></Button>
            </Tooltip>
          </Dropdown>
          <Button
            type="text"
            size="small"
            icon={<CloseOutlined />}
            onClick={() => props.onClose?.()}
          ></Button>
        </span>
      </div>

      <div className="text-14px pl-12px text-#333">绘制图形</div>
      <div
        className="chat-content bg-#f5f5f5 px-10px overflow-y-auto mt-12px"
        ref={scrollAreaRef}
      >
        <div
          style={{
            borderColor: focused ? "#1890ff" : "#ddd",
          }}
          className="chat-foot bg-#fff rounded-10px border border-solid border-1px shadow-sm"
        >
          <Dropdown menu={dropDownMenu} placement="bottomLeft">
            <div className="text-12px pl-10px pt-10px">
              帮我绘制-流程图
              <CaretDownOutlined />
            </div>
          </Dropdown>
          <Form onFinish={onSubmit}>
            <Input.TextArea
              rows={3}
              autoSize={{ maxRows: 3, minRows: 3 }}
              placeholder="你可以这样问：用户登陆流程图"
              variant="borderless"
              onFocus={() => setFocused(true)}
              onBlur={() => setFocused(false)}
              value={input}
              onChange={(e) => setInput(e.target.value)}
              disabled={loading}
              onPressEnter={onSubmit}
            />
            <div className="text-right p-10px">
              {loading ? (
                <Tooltip title="停止生成">
                  <Button
                    type="primary"
                    shape="circle"
                    icon={<i className="iconfont icon-stopcircle" />}
                    onClick={stop}
                  ></Button>
                </Tooltip>
              ) : (
                <Button
                  type="text"
                  icon={<SendOutlined />}
                  disabled={!input.trim()}
                  htmlType="submit"
                ></Button>
              )}
            </div>
          </Form>
        </div>
      </div>

      <div className="text-14px pl-12px text-#333 mt-32px">图形处理</div>
      <div className="flex flex-wrap gap-10px p-10px">
        {handleList.map((item) => (
          <div
            key={item.key}
            className="flex-[40%] h-50px bg-#fff rounded-10px shadow-sm flex items-center pl-10px text-12px cursor-pointer"
            style={{}}
          >
            <i
              className={`iconfont ${item.icon} text-16px`}
              style={{ color: item.color }}
            ></i>
            <span className="ml-10px">{item.label}</span>
          </div>
        ))}
      </div>
    </div>
  );
}
